#include <iostream>
#include <ctime>
#include <cstdlib>
#include <cstring>
#include <cmath>

using namespace std;

int main()
{
    srand(time(NULL));
    while(true)
    {
        while(true)
        {
            float balance(0);
            float balanceHigh(balance);
            cout << "Welcome to Kanzler's martingale simulation. This program is designed to demonstrate expectancy of the martingale betting system - by using it, you will be able to see first-hand the results over many simulations without risking any money or messing with an EA. You may use, change, and share this basic software free of charge, with the following conditions - you do not remove this notice, and you do not sell this software or portions thereof for personal or corporate gain." << endl << endl;

            cout << "Please enter an initial account balance from 1 to 1000 units:  ";
            cin >> balance;
            if(balance < 1 || balance > 1000)
            {
                cout << "Sorry, your input is invalid." << endl << endl;
                break;
            }
            cout << "Your account balance is $" << balance << ". Please enter a value up to 100 as your percentage risk: ";
            float originalBalance = balance;
            float percent(0);
            cin >> percent;
            if(percent > 100)
            {
                cout << "Sorry, your input is invalid." << endl << endl;
                break;
            }
            percent = percent * 0.01;
            cout << "Your default risk is " << percent*100 << " percent per trade." << endl;
            cout << "Please enter your average probability of winning from 0 to 100 percent (a \"win\" is reaching 1x risk and cashing out): ";
            float probability(0);
            cin >> probability;
            if(probability < 0 || probability > 100)
            {
                cout << "Sorry, your input is invalid." << endl << endl;
                break;
            }
            bool showTrades(false);
            char boolean[2];
            memset(boolean, 2, 0);
            cout << "How many simulations should be executed? (any positive integer): ";
            unsigned int runs(0);
            cin >> runs;
            if(runs < 0)
            {
                cout << "Sorry, your input is invalid." << endl << endl;
                break;
            }
            cout << "View individual trades? [Y/N]: ";
            cin >> boolean;
            if((strcmp(boolean, "Y") == 0)
                || strcmp(boolean, "y") == 0)
            {
                showTrades = true;
            }
            else if((strcmp(boolean, "N") == 0)
                    || (strcmp(boolean, "n") == 0))
            {
                showTrades = false;
            }
            else
            {
                cout << "Sorry, your input is invalid." << endl << endl;
                break;
            }
            bool martingale(false);
            cout << "Use martingale betting system? [Y/N]: ";
            memset(boolean, 2, 0);
            cin >> boolean;
            if((strcmp(boolean, "Y") == 0)
                || strcmp(boolean, "y") == 0)
            {
                martingale = true;
            }
            else if((strcmp(boolean, "N") == 0)
                    || (strcmp(boolean, "n") == 0))
            {
                martingale = false;
            }
            else
            {
                cout << "Sorry, your input is invalid." << endl << endl;
                break;
            }
            cout << "Hit enter to run simulation(s)." << endl;
            float totalReturns(0);
            for(unsigned int i(0); i<runs; i++)
            {
                unsigned int counter(0);
                bool outcome(false);
                unsigned int random(0);
                float actualRisk(0);
                float martingalePercent(percent);
                if(showTrades)
                    cout << "[Balance]" << balance << endl;
                while(true)
                {
                    counter++;
                    random = rand()%100;
                    actualRisk = balance*(martingalePercent);
                    if(random < probability)
                    {
                        outcome = true;
                        balance = balance+actualRisk;
                    }
                    else
                    {
                        outcome = false;
                        balance = balance-actualRisk;
                    }
                    balance = floor(balance * 100.0) / 100.0;
                    if(balance > balanceHigh)
                        balanceHigh = balance;
                    if(showTrades)
                        cout << "[Trade " << counter << "][";
                    if(outcome == true)
                    {
                        if(showTrades)
                            cout << "W][R: " << martingalePercent << " (+$" << actualRisk << ")] $" << balance << endl;
                        martingalePercent = percent;    /// Reset risk on winner.
                    }
                    else
                    {
                        if(showTrades)
                            cout << "L][R: " << martingalePercent << " (-$" << actualRisk << ")] $" << balance << endl;
                        if(martingale)
                            martingalePercent = martingalePercent * 2;    /// Double risk on loser.
                        else
                            martingalePercent = percent;
                        if(martingalePercent > 1)
                            martingalePercent = 1;
                    }
                    if(showTrades)
                        cin.ignore();
                    if(balance <= 0)
                    {
                        cout << "Bankrupt after " << counter << " trades at original " << percent<< " risk. The high balance for your account was $" << balanceHigh << "." << endl;
                        totalReturns += balanceHigh;
                        balance = originalBalance;
                        balanceHigh = balance;
                        break;
                    }
                    else if(counter > 10000)
                    {
                        cout << "Maximum number of trades exceeded. Account balance is currently " << balance << " with a high balance of " << balanceHigh << "." << endl;
                        totalReturns += balanceHigh;
                        balance = originalBalance;
                        balanceHigh = balance;
                        break;
                    }
                }
            }
            cout << "If you had cashed out at the peak for each run, you would have averaged $" << (float)totalReturns/runs << " per try without factoring in spread or fees. But you didn't know where this peak was at, so you lost everything." << endl << endl;
        }
    }
    return 0;
}
